//
//  AstroImage.java
//  Affichage d'une image avec les coordonnes astronomiques
//

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.image.*;
import java.util.ResourceBundle;

public class AstroImage extends Applet {

    static ResourceBundle rb  = ResourceBundle.getBundle("LocalStrings");
    ImageCanvas ic;
    Label lab;
    
    public void init() {
        setLayout(new BorderLayout());
        String nomImage = getParameter("image");
        String sa1 = getParameter("a1");
        String sa2 = getParameter("a2");
        String sd1 = getParameter("d1");
        String sd2 = getParameter("d2");
        if  (nomImage == null || sa1 == null || sa2 == null || sd1 == null || sd2 == null) {
            System.err.println("Il manque un des paramtres de l'applet");
            return;
        }
        double a1,a2,d1,d2;
        try {
            a1 = stringToCoord(sa1);
            a2 = stringToCoord(sa2);
            d1 = stringToCoord(sd1);
            d2 = stringToCoord(sd2);
        } catch (NumberFormatException ex) {
            System.err.println("Coordonnes mal formattes. Format RA/DEC: +xx:xx:xx.xxx");
            return;
        }
        Image im = getImage(getCodeBase(), nomImage);
        MediaTracker tracker = new MediaTracker(this);
        tracker.addImage(im, 0);
        try {
            tracker.waitForAll();
        } catch (InterruptedException ex) {
            System.err.println("VisImage: InterruptedException: " + ex.getMessage());
        }
        ic = new ImageCanvas(im, a1, a2, d1, d2);
        ic.addMouseMotionListener(ic);
        add(ic, BorderLayout.CENTER);
        lab = new Label(" ");
        add(lab, BorderLayout.SOUTH);
        validate();
    }
    
    public static double stringToCoord(String s) throws NumberFormatException {
        int ind1, ind2;
        String s1,s2,s3;
        double d1,d2,d3;
        int signe;
        
        ind1 = s.indexOf(':');
        if (ind1 == -1)
            throw new NumberFormatException();
        ind2 = s.substring(ind1 + 1).indexOf(':');
        if (ind2 == -1)
            throw new NumberFormatException();
        ind2 += ind1 + 1;
        
        s1 = s.substring(0, ind1);
        s2 = s.substring(ind1 + 1, ind2);
        s3 = s.substring(ind2 + 1);
        
        d1 = (new Double(s1)).doubleValue();
        d2 = (new Double(s2)).doubleValue();
        d3 = (new Double(s3)).doubleValue();
        
        if (d1 < 0)
            signe = -1;
        else
            signe = 1;
        
        return(d1 + signe*d2/60 + signe*d3/3600);
    }
    
    public static String coordToString(double c, boolean ra) {
        String s1,s2,s3;
        double d1,d2,d3;
        String signe;
        
        if (c < 0) {
            signe = "-";
            c = -c;
        } else
            signe = "";
        
        d1 = Math.floor(c);
        d2 = Math.floor(Math.rint((c - d1) * 3600)/60);
        d3 = Math.rint(((c - d1) * 60 - d2) * 60);
        
        s1 = String.valueOf((int)d1);
        s2 = String.valueOf((int)d2);
        if (s2.length() == 1)
            s2 = "0" + s2;
        s3 = String.valueOf((int)d3);
        if (s3.length() == 1)
            s3 = "0" + s3;
        
        if (ra)
            return(signe + s1 + "h" + s2 + "m" + s3 + "s");
        else
            return(signe + s1 + "" + s2 + "'" + s3 + "\"");
    }
    
    class ImageCanvas extends Canvas implements MouseMotionListener {
        Image image;
        Dimension size;
        int w, h;
        int wleft = 60;
        int hbottom = 50;
        double a1,a2,d1,d2;
    
        // on suppose ici que l'image est dj charge
        public ImageCanvas(Image image, double a1, double a2, double d1, double d2) {
            
            if (image == null) {
                System.err.println("AstroImage.ImageCanvas: erreur: image == null");
                return;
            }
    
            this.image = image;
            
            this.a1 = a1;
            this.a2 = a2;
            this.d1 = d1;
            this.d2 = d2;
    
            w = image.getWidth(null);
            h = image.getHeight(null);
    
            if (w == -1 || h == -1) {
                System.err.println("AstroImage.ImageCanvas: erreur: image non charge");
                w = 100;
                h = 100;
            }
            
            w = w + wleft;
            h = h + hbottom;
    
            size = new Dimension(w, h);

            setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
        }
    
        public Dimension getPreferredSize() {
            return getMinimumSize();
        }
    
        public Dimension getMinimumSize() {
            return size;
        }
    
        public void paint (Graphics g) {
            if (image != null)
                g.drawImage(image, wleft, 0, this);
            
            g.drawRect(wleft, 0, w - wleft - 1, h - hbottom - 1);
            
            Font defonce = g.getFont();
            Font plafond = new Font(defonce.getName(), Font.PLAIN, 10);
            g.setFont(plafond);
            Font boldfont = new Font(defonce.getName(), Font.BOLD, 10);
            
            // AD
            
            int azoom = 1;
            if (Math.abs(a2 - a1) < 1.0)
                azoom = 2; // affiche les minutes
            if (Math.abs(a2 - a1) < 1.0/60)
                azoom = 3; // affiche les secondes
            
            if (azoom == 1) {
                // les heures
                grilleAD(g, 1);
            }
            if (azoom == 2) {
                // les minutes
                grilleAD(g, 60);
            }
            if (azoom == 3) {
                // les secondes
                grilleAD(g, 3600);
            }
            g.setFont(boldfont);
            String sad = rb.getString("Ascension_Droite");
            g.drawString(sad, wleft + (w - wleft)/2 - (g.getFontMetrics().stringWidth(sad))/2, h - hbottom/4);
            g.setFont(plafond);
            
            // DEC
            
            int dzoom = 1;
            if (Math.abs(d2 - d1) < 1.0)
                dzoom = 2; // affiche les minutes
            if (Math.abs(d2 - d1) < 1.0/60)
                dzoom = 3; // affiche les secondes
            
            if (dzoom == 1) {
                // les degrs
                grilleDEC(g, 1);
            }
            if (dzoom == 2) {
                // les minutes
                grilleDEC(g, 60);
            }
            if (dzoom == 3) {
                // les secondes
                grilleDEC(g, 3600);
            }
            g.setFont(boldfont);
            String sdec = rb.getString("DEC");
            g.drawString(sdec, 2, (h - hbottom)/2);
            g.setFont(plafond);

        }
        
        public void grilleAD(Graphics g, double facteur) {
            int ia1, ia2, px;
            String scoord;
            
            if (a2 > a1) {
                ia1 = (int)Math.ceil(a1*facteur);
                ia2 = (int)Math.floor(a2*facteur);
            } else {
                ia1 = (int)Math.ceil(a2*facteur);
                ia2 = (int)Math.floor(a1*facteur);
            }
            for (int i=ia1; i<=ia2; i++)
                if (Math.abs(a2 - a1) < 10/facteur || (10*(i/10) == i)) {
                    if (a2 > a1)
                        px = (int)Math.rint(((i/facteur - a1) / (a2 - a1)) * (w - wleft)) + wleft;
                    else
                        px = w - (int)Math.rint(((i/facteur - a2) / (a1 - a2)) * (w - wleft));
                    scoord = coordToString(i/facteur, true);
                    g.drawString(scoord, px - (g.getFontMetrics().stringWidth(scoord))/2, h - hbottom/2);
                    g.setXORMode(Color.white);
                    g.drawLine(px, 0, px, h - hbottom);
                    g.setPaintMode();
                }
        }
        
        public void grilleDEC(Graphics g, double facteur) {
            int id1, id2, py;
            String scoord;
            
            if (d2 > d1) {
                id1 = (int)Math.ceil(d1*facteur);
                id2 = (int)Math.floor(d2*facteur);
            } else {
                id1 = (int)Math.ceil(d2*facteur);
                id2 = (int)Math.floor(d1*facteur);
            }
            for (int i=id1; i<=id2; i++)
                if (Math.abs(d2 - d1) < 10.0/facteur || (10*(i/10) == i)) {
                    if (d2 > d1)
                        py = (int)Math.rint(((i/facteur - d1) / (d2 - d1)) * (h - hbottom));
                    else
                        py = h - hbottom - (int)Math.rint(((i/facteur - d2) / (d1 - d2)) * (h - hbottom));
                    scoord = coordToString(i/facteur, false);
                    g.drawString(scoord, 10, py);
                    g.setXORMode(Color.white);
                    g.drawLine(wleft, py, w, py);
                    g.setPaintMode();
                }
        }
        
        public void mouseMoved(MouseEvent e) {
            if (e.getX() > wleft && e.getY() < h - hbottom) {
                double a = ((e.getX() - wleft) * 1.0 / (w - wleft)) * (a2 - a1) + a1;
                double d = (e.getY() * 1.0 / (h - hbottom)) * (d2 - d1) + d1;
                lab.setText(" " + coordToString(a, true) + ", " + coordToString(d, false));
            } else
                lab.setText(" ");
        }
        
        public void mouseDragged(MouseEvent e) {
        }
        
        /*public void changeImage(Image im) {
            tracker.removeImage(image,0);
            this.image = im;
            tracker.addImage(image, 0);
        }*/
    }

}
