/*

Copyright (C) 2012  Observatoire de Paris

The JavaScript code in this page is free software: you can
redistribute it and/or modify it under the terms of the GNU
General Public License (GNU GPL) as published by the Free Software
Foundation, either version 3 of the License, or (at your option)
any later version.  The code is distributed WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.

As additional permission under GNU GPL version 3 section 7, you
may distribute non-source (e.g., minimized or compacted) forms of
that code without the copy of the GNU GPL normally required by
section 4, provided you include this license notice and a URL
through which recipients can access the Corresponding Source.

*/

var image;
var canvas;
var context;
var wleft = 60;
var hbottom = 50;
var w, h; // dimensions du canvas
var a1, a2, d1, d2; // coordonnées sous forme de nombres

function lireParametres() {
    var parametres = new Array();
    var searchString = window.location.search.substring(1);
    var params = searchString.split("&");
    
    for (var i=0; i<params.length; i++) {
        var nomval = params[i].split("=");
        // remarque: les paramètres de l'URL doivent être encodés en UNICODE
        parametres[nomval[0]] = decodeURIComponent(nomval[1]);
    }
    return(parametres);
}

function init() {
    verifSafari4();
    var parametres = lireParametres();
    var chemin_image = parametres['image'];
    var sa1 = parametres['a1'];
    var sa2 = parametres['a2'];
    var sd1 = parametres['d1'];
    var sd2 = parametres['d2'];
    if (chemin_image == null || sa1 == null || sa2 == null || sd1 == null || sd2 == null) {
        alert("Paramètres obligatoires: image, a1, a2, d1, d2");
        return;
    }
    var chemin_fichiers = parametres['chemin_fichiers'];
    if (chemin_fichiers != null)
        chemin_image = chemin_fichiers + chemin_image;
    a1 = stringToCoord(sa1);
    a2 = stringToCoord(sa2);
    d1 = stringToCoord(sd1);
    d2 = stringToCoord(sd2);
    image = new Image();
    image.onload = premierAffichage;
    image.onerror = function(err) { alert("Erreur au chargement de l'image " + chemin_image) };
    image.src = chemin_image;
}

function verifSafari4() {
    // fillText ne marche pas avec Safari avant la version 4, et on ne peut pas utiliser FlashCanvas avec Safari...
    // Une solution serait d'utiliser canvas.text.js, mais ça ne marche pas bien...
    if (navigator.userAgent && navigator.userAgent.indexOf('AppleWebKit') > 0) {
        var index = navigator.userAgent.indexOf('Version/');
        if (index > 0) {
            var sv = navigator.userAgent.substring(index + 'Version/'.length);
            index = sv.indexOf(' ');
            if (index > 0) {
                sv = sv.substring(0, index);
                if (sv < '4.0')
                    alert('Cette application ne fonctionne pas avec Safari avant la version 4.0. Veuillez le mettre à jour ou essayer avec Firefox.');
            }
        }
    }
}

function premierAffichage() {
    var body = document.getElementsByTagName('body')[0];
    canvas = document.createElement('canvas');
    w = image.width + wleft;
    canvas.setAttribute('width', w);
    h = image.height + hbottom;
    canvas.setAttribute('height', h);
    body.appendChild(canvas);
    if (typeof G_vmlCanvasManager  != 'undefined' ) {
        G_vmlCanvasManager.initElement(canvas);
    }
    if (canvas.addEventListener)
        canvas.addEventListener('mousemove', mousemove, false);
    else
        canvas.attachEvent('onmousemove', mousemove);
    if ('ontouchstart' in document) {
        canvas.addEventListener('touchstart', mousemove, false);
        canvas.addEventListener('touchmove', mousemove, false);
    }
    
    if (!canvas.getContext) {
        body.innerHTML = '<p>Votre navigateur ne gère pas HTML5.</p>';
        return;
    }
    context = canvas.getContext('2d');
    paint();
}

function paint() {
    context.drawImage(image, wleft, 0, image.width, image.height);
    //context.strokeStyle = 'black';
    //context.strokeRect(wleft, 0, w - wleft - 1, h - hbottom - 1);
    
    // AD
    
    context.font = 'normal 10px sans-serif'; // il faut ajouter normal pour éviter un bug de FlashCanvas
    context.textAlign = 'left';
    context.textBaseline = 'alphabetic';
    var azoom = 1;
    if (Math.abs(a2 - a1) < 1.0)
        azoom = 2; // affiche les minutes
    if (Math.abs(a2 - a1) < 1.0/60)
        azoom = 3; // affiche les secondes
    
    if (azoom == 1) {
        // les heures
        grilleAD(1);
    }
    if (azoom == 2) {
        // les minutes
        grilleAD(60);
    }
    if (azoom == 3) {
        // les secondes
        grilleAD(3600);
    }
    context.fillStyle = 'black';
    context.font = 'bold 10px sans-serif';
    var sad = 'Ascension Droite';
    var metrics = context.measureText(sad);
    context.fillText(sad, wleft + (w - wleft)/2 - metrics.width/2, h - hbottom/4);
    context.font = 'normal 10px sans-serif';
    
    // DEC
    
    var dzoom = 1;
    if (Math.abs(d2 - d1) < 1.0)
        dzoom = 2; // affiche les minutes
    if (Math.abs(d2 - d1) < 1.0/60)
        dzoom = 3; // affiche les secondes
    
    if (dzoom == 1) {
        // les degrés
        grilleDEC(1);
    }
    if (dzoom == 2) {
        // les minutes
        grilleDEC(60);
    }
    if (dzoom == 3) {
        // les secondes
        grilleDEC(3600);
    }
    context.font = 'bold 10px sans-serif';
    var sdec = 'Déc.';
    context.fillText(sdec, 2, (h - hbottom)/2);
    context.font = 'normal 10px sans-serif';
}

function grilleAD(facteur) {
    var ia1, ia2, px;
    var scoord;
    
    context.save();
    if (a2 > a1) {
        ia1 = Math.ceil(a1*facteur);
        ia2 = Math.floor(a2*facteur);
    } else {
        ia1 = Math.ceil(a2*facteur);
        ia2 = Math.floor(a1*facteur);
    }
    for (var i=ia1; i<=ia2; i++)
        if (Math.abs(a2 - a1) < 10/facteur || (i % 10 == 0)) {
            if (a2 > a1)
                px = Math.round(((i/facteur - a1) / (a2 - a1)) * (w - wleft)) + wleft; // Math.rint
            else
                px = w - Math.round(((i/facteur - a2) / (a1 - a2)) * (w - wleft)); // Math.rint
            scoord = coordToString(i/facteur, true);
            var metrics = context.measureText(scoord);
            // pas de moyen simple pour faire une ligne XOR, ni pour faire une ligne en pointillés,
            // on essaye avec une ligne blanche et une ligne noire...
            context.fillStyle = 'black';
            context.fillText(scoord, px - metrics.width/2 + 0.5, h - hbottom + 15);
            context.beginPath();
            context.strokeStyle = 'white';
            context.moveTo(px-0.5, 0);
            context.lineTo(px-0.5, h - hbottom);
            context.stroke();
            context.beginPath();
            context.strokeStyle = 'black';
            context.moveTo(px+0.5, 0);
            context.lineTo(px+0.5, h - hbottom);
            context.stroke();
        }
    context.restore();
}

function grilleDEC(facteur) {
    var id1, id2, py;
    var scoord;
    
    context.save();
    if (d2 > d1) {
        id1 = Math.ceil(d1*facteur);
        id2 = Math.floor(d2*facteur);
    } else {
        id1 = Math.ceil(d2*facteur);
        id2 = Math.floor(d1*facteur);
    }
    for (var i=id1; i<=id2; i++)
        if (Math.abs(d2 - d1) < 10.0/facteur || (i % 10 == 0)) {
            if (d2 > d1)
                py = Math.round(((i/facteur - d1) / (d2 - d1)) * (h - hbottom)); // Math.rint
            else
                py = h - hbottom - Math.round(((i/facteur - d2) / (d1 - d2)) * (h - hbottom)); // Math.rint
            scoord = coordToString(i/facteur, false);
            context.fillStyle = 'black';
            context.fillText(scoord, 5, py + 2);
            context.beginPath();
            context.strokeStyle = 'white';
            context.moveTo(wleft, py-0.5);
            context.lineTo(w, py-0.5);
            context.stroke();
            context.beginPath();
            context.strokeStyle = 'black';
            context.moveTo(wleft, py+0.5);
            context.lineTo(w, py+0.5);
            context.stroke();
        }
    context.restore();
}

function coordonnees(e) {
    var x, y;
    if (e.pageX || e.pageY) {
      x = e.pageX;
      y = e.pageY;
    } else {
      x = e.clientX + document.body.scrollLeft + document.documentElement.scrollLeft;
      y = e.clientY + document.body.scrollTop + document.documentElement.scrollTop;
    }
    var totalOffsetX = 0;
    var totalOffsetY = 0;
    var currentElement = canvas;
    do {
        totalOffsetX += currentElement.offsetLeft;
        totalOffsetY += currentElement.offsetTop;
    } while (currentElement = currentElement.offsetParent)
    x -= totalOffsetX;
    y -= totalOffsetY;
    return( { x : x, y : y } );
}

function mousemove(e) {
    context.save();
    var pt = coordonnees(e);
    context.fillStyle = 'white';
    var metrics = context.measureText('HHhMMmSSs, -DD°MM\'SS"');
    context.fillRect(20, h - hbottom/2, metrics.width + 5, hbottom/2);
    if (pt.x > wleft && pt.y < h - hbottom) {
        var a = ((pt.x - wleft) * 1.0 / (w - wleft)) * (a2 - a1) + a1;
        var d = (pt.y * 1.0 / (h - hbottom)) * (d2 - d1) + d1;
        context.fillStyle = 'black';
        var s = coordToString(a, true) + ', ' + coordToString(d, false);
        context.fillText(s, 20, h - 5, 200);
    }
    context.restore();
    if (e.preventDefault)
        e.preventDefault();
    else
        e.returnValue = false;
}

function stringToCoord(s) {
    var ind1, ind2;
    var s1,s2,s3;
    var d1,d2,d3;
    var signe;
    
    ind1 = s.indexOf(':');
    if (ind1 == -1)
        return(NaN);
    ind2 = s.substring(ind1 + 1).indexOf(':');
    if (ind2 == -1)
        return(NaN);
    ind2 += ind1 + 1;
    
    s1 = s.substring(0, ind1);
    s2 = s.substring(ind1 + 1, ind2);
    s3 = s.substring(ind2 + 1);
    
    d1 = Number(s1);
    d2 = Number(s2);
    d3 = Number(s3);
    
    if (d1 < 0)
        signe = -1;
    else
        signe = 1;
    
    return(d1 + signe*d2/60 + signe*d3/3600);
}

function coordToString(c, ra) {
    var s1,s2,s3;
    var d1,d2,d3;
    var signe;
    
    if (c < 0) {
        signe = '-';
        c = -c;
    } else
        signe = '';
    
    d1 = Math.floor(c);
    d2 = Math.floor(Math.round((c - d1) * 3600)/60); // Math.floor(Math.rint)
    d3 = Math.round(((c - d1) * 60 - d2) * 60); // Math.rint
    
    s1 = '' + d1;
    s2 = '' + d2;
    if (s2.length == 1)
        s2 = '0' + s2;
    s3 = '' + d3;
    if (s3.length == 1)
        s3 = '0' + s3;
    
    if (ra)
        return(signe + s1 + 'h' + s2 + 'm' + s3 + 's');
    else
        return(signe + s1 + '°' + s2 + "'" + s3 + '"');
}

